
import { isNullOrUndefined } from 'util';

var checkValueType = require('./Utils').checkValueType;
var checkParamCount = require('./Utils').checkParamCount;
var reverse = require('./Utils').reverse;
var removeNullParam = require('./Utils').removeNullParam;
var replaceNullParam = require('./Utils').replaceNullParam;
var SimpleDateFormat = require('./Utils').SimpleDateFormat
var getFirstNum = require('./Utils').getFirstNum;
var checkReg = require('./Utils').checkReg;
var checkDateValid = require('./Utils').checkDateValid;
var tofixed = require('./Utils').tofixed;
var moment = require('moment');
var md5 = require('md5');
var CryptoJS = require('crypto-js')
/**
 * 函数的定义
 */
export class Function {
  // 获取一般方法的列表
  getFuncMap() {
    return {
      'IF': this.funcIf,
      'CONCAT': this.funcConcat,
      'LEFT': this.funcLeft,
      'RIGHT': this.funcRight,
      'REPLACE': this.funcReplace,
      'MID': this.funcMid,
      'TEXT': this.funcText,
      'JOIN': this.funcJoin,
      'NUM': this.funNum,
      'RMBUPPER': this.funRmbUpper,
      'SUM': this.funcSum,
      'SUMIF': this.funcSumIf,
      'AVERAGE': this.funcAverage,
      'COUNT': this.funcCount,
      'COUNTIF': this.funcCountIf,
      'MIN': this.funcMin,
      'MAX': this.funcMax,
      'ROUND': this.funcRound,
      'ROUNDUP': this.funcRoundUp,
      'ROUNDDOWN': this.funcRoundDown,
      'INT': this.funcInt,
      'MOD': this.funcMod,
      'PRODUCT': this.funcProduct,
      'SUMPRODUCT': this.funcSumProduct,
      'POWER': this.funcPower,
      'YEAR': this.funcYear,
      'MONTH': this.funcMonth,
      'DAY': this.funcDay,
      'HOUR': this.funcHour,
      'MINUTE': this.funcMinite,
      'SECOND': this.funcSecond,
      'DATE': this.funcDate,
      'CURDATE': this.funcCurDate,
      'NOW': this.funcNow,
      'RDID': this.funcRDID,
      'UPPER': this.funcUpper,
      'LOWER': this.funcLower,
      'AND': this.funcAnd,
      'OR': this.funcOr,
      'NOT': this.funcNot,
      'XOR': this.funcXor,
      'ISEMPTY': this.funcIsEmpty,
      'SEARCH': this.funcSearch,
      'LEN': this.funcLen,
      'DAYS': this.funcDays,
      'DATEDELTA': this.funcDateDelta,
      'REGTEST': this.funcRegTest,
      'LN': this.funcLn,
      'IFS': this.funcIfs,
      'BASE64': this.funcBase64,
      'MD5': this.funcMd5,
      'TRIPLEDES': this.funTRIPLEDES,
      'SQRT': this.funcSqrt,
      'DATETOTIMESTAMP': this.funcDateToTimestamp,
      'TIMESTAMPTODATE': this.funcTimestampToDate,
      'NETWORKDAYS': this.funcNetWorkdays,
      'UNICODEENCODE': this.funcUnicodeEncode,
      'UNICODEDECODE': this.funcUnicodeDecode
    }
  }

  // 获取所有需要用户信息来计算的方法
  getFuncMapWithParam() {
    return {
      'GETUSERNAME': this.funcGetUserName,
      'GETUSEREMAIL': this.funcGetUserEmail,
      'JAID': this.funJAaId,
      'JANAME': this.funJaName,
      'JATYPE': this.funJaType,
      'JADEPTID': this.funJaDeptId,
      'JADEPTNAME': this.funJaDeptName,
      'RECNO': this.funcRecno
    }
  }

  // 获取成员部门等特殊信息来计算的方法
  getFuncMapWithDetailInfos() {
    return {
      'TEXTUSER': this.funcTextUser,
      'TEXTDEPT': this.funcTextDept
    }
  }

  // if表达式
  funcIf(test, value1, value2) {
    checkValueType('boolean', 'IF', 0, test);
    checkParamCount('IF', 3, arguments);
    return test ? value1 : value2;
  }
  // 把字符串串起来
  funcConcat(...values) {
    values = [].concat(...values); // flat
    return values.join('');
  }
  // 取左边的n个数
  funcLeft(value, n) {
    checkValueType('number', 'LEFT', 1, n);
    checkParamCount('LEFT', 2, arguments);
    value = String(value);
    return value.substr(0, n);
  }
  // 取右边的n个数
  funcRight(value, n) {
    checkValueType('number', 'RIGHT', 1, n);
    checkParamCount('RIGHT', 2, arguments);
    return reverse(reverse(value).substr(0, n));
  }
  // 取代从start开始的长度为n的位置的字符串（注意，用户使用时，第一个位置是1，而不是0）
  funcReplace(value, start, n, newValue) {
    value = String(value);
    newValue = String(newValue);
    checkValueType('number', 'REPLACE', 1, start, n);
    checkParamCount('REPLACE', 4, arguments);
    return value.substring(0, start - 1) + newValue + value.substr(start + n - 1);
  }
  // 取中间的子串，从start开始，取长度为n的子串
  funcMid(value, start, n) {
    checkParamCount('MID', 3, arguments);
    value = String(value);
    return value.substr(start - 1, n);
  }
  // 把时间转化为字符串
  funcText(value, timeFormat) {
    checkParamCount('TEXT', 2, arguments);
    if (timeFormat === 'E') {
      moment.locale('en');
      return moment(value).format('e');
    }
    else if (timeFormat === 'EE') {
      moment.locale('zh-cn');
      return moment(value).format('ddd');
    }
    else if (timeFormat === 'EEE') {
      moment.locale('zh-cn');
      return moment(value).format('dddd');
    }
    else if (timeFormat) {
      var t = moment(value, ['YYYY-MM-DD', 'YYYY-MM-DD HH:mm:ss']);
      return t.format(timeFormat);
    } else {
      return `"${value}"`;
    }
  }

  // 连接单值、多值字段
  funcJoin() {
    const [connector, ...args] = arguments;
    const valueArray = [];
    // 最少得有2个参数
    if (arguments.length < 2) {
      checkParamCount('JOIN', 2, arguments, );
    }
    args.forEach((queValue) => {
      if (Array.isArray(queValue)) {
        valueArray.push(...queValue)
      } else {
        valueArray.push(queValue)
      }
    })
    return valueArray.join(connector);
  }

  // 把其他类型的字段转换成数字类型，可以识别浮点类型
  funNum(...values) {
    values = [].concat(...values);
    if (values.length < 2) {
      return getFirstNum(values[0]);
    } else {
      return values.map(val => getFirstNum(val));
    }
  }

  // 数字转化为人民币大写形式
  funRmbUpper(value) {
    //汉字的数字
    var cnNums = new Array('零', '壹', '贰', '叁', '肆', '伍', '陆', '柒', '捌', '玖');
    //基本单位
    var cnIntRadice = new Array('', '拾', '佰', '仟');
    //对应整数部分扩展单位
    var cnIntUnits = new Array('', '万', '亿', '兆');
    //对应小数部分单位
    var cnDecUnits = new Array('角', '分', '毫', '厘');
    //整数金额时后面跟的字符
    var cnInteger = '整';
    //整型完以后的单位
    var cnIntLast = '元';
    //最大处理的数字
    var maxNum = 999999999999999.9999;
    //金额整数部分
    var integerNum;
    //金额小数部分
    var decimalNum;
    //输出的中文金额字符串
    var chineseStr = '';
    //分离金额后用的数组，预定义
    var parts;
    if (value == '') { return ''; }
    value = parseFloat(value);
    if (value >= maxNum) {
      //超出最大处理数字
      return '';
    }
    if (value == 0) {
      chineseStr = cnNums[0] + cnIntLast + cnInteger;
      return chineseStr;
    }
    //转换为字符串
    value = value.toString();
    if (value.indexOf('.') == -1) {
      integerNum = value;
      decimalNum = '';
    } else {
      parts = value.split('.');
      integerNum = parts[0];
      decimalNum = parts[1].substr(0, 4);
    }
    //获取整型部分转换
    if (parseInt(integerNum, 10) > 0) {
      var zeroCount = 0;
      var IntLen = integerNum.length;
      for (var i = 0; i < IntLen; i++) {
        var n = integerNum.substr(i, 1);
        var p = IntLen - i - 1;
        var q = p / 4;
        var m = p % 4;
        if (n == '0') {
          zeroCount++;
        } else {
          if (zeroCount > 0) {
            chineseStr += cnNums[0];
          }
          //归零
          zeroCount = 0;
          chineseStr += cnNums[parseInt(n)] + cnIntRadice[m];
        }
        if (m == 0 && zeroCount < 4) {
          chineseStr += cnIntUnits[q];
        }
      }
      chineseStr += cnIntLast;
    }
    //小数部分
    if (decimalNum != '') {
      var decLen = decimalNum.length;
      for (var i = 0; i < decLen; i++) {
        var n = decimalNum.substr(i, 1);
        if (n != '0') {
          chineseStr += cnNums[Number(n)] + cnDecUnits[i];
        }
      }
    }
    if (chineseStr == '') {
      chineseStr += cnNums[0] + cnIntLast + cnInteger;
    } else if (decimalNum == '') {
      chineseStr += cnInteger;
    }
    return chineseStr;
  }
  // 求和
  funcSum(...values) {
    values = [].concat(...values); // flat
    // 如果没有值的话，返回0
    if (values.length === 0) {
      return 0;
    }
    values = replaceNullParam(values);
    // 类型检查
    checkValueType('number', 'SUM', 0, values);
    return values.reduce((pre, next) => {
      return Number.parseFloat(pre).add(Number.parseFloat(next));
    })
  }
  // 满足条件的情况下再求和
  funcSumIf(base, compare, values) {
    // 如果values为空或者判定条件为空
    if (isNullOrUndefined(base) || isNullOrUndefined(values) || base.length === 0 || values.length === 0) {
      return 0;
    }
    // 需要求和的值，如果有空字符串，null或者undefined，直接赋值为0
    values.forEach((v, index) => {
      if (isNullOrUndefined(v) || v === "") {
        values.splice(index, 1, 0);
      }
    })
    // 类型检查
    checkValueType('number', 'SUMIF', 2, values);
    // compare传递的是一个数组时，对数组中每一个值求对比结果
    if (!isNullOrUndefined(compare) && compare.constructor.name === 'Array') {
      let result = [];
      compare.forEach(compareVal => {
        let tmpResult = 0;
        base.forEach((baseVal, index) => {
          if (baseVal === compareVal && index < values.length) {
            // tmpResult += values[index];
            tmpResult = tmpResult.add(values[index])
          }
        })
        result.push(tmpResult);
      })
      return result;
    } else {
      let result = 0;
      base.forEach((baseVal, index) => {
        if (baseVal === compare && index < values.length) {
          // result += values[index];
          result = result.add(values[index]);
        }
      })
      return result;
    }
  }


  // 平均数
  funcAverage(...values) {
    values = [].concat(...values); // flat
    // 如果没有值的话，返回0
    if (values.length === 0) {
      return 0;
    }
    values = replaceNullParam(values);
    // 类型检查
    checkValueType('number', 'AVERAGE', 0, values);
    var sum = values.reduce((pre, next) => {
      //   return Number.parseFloat(pre) + Number.parseFloat(next);
      return Number.parseFloat(pre).add(Number.parseFloat(next));
    })
    //   return sum / values.length;
    return sum.div(values.length);
  }
  // 计数
  funcCount(...values) {
    values = [].concat(...values); // flat
    values = removeNullParam(values);
    return values.length;
  }
  // 根据条件计数
  funcCountIf(values, compareVal) {
    if (isNullOrUndefined(values) || values.length === 0) {
      return 0;
    }
    if (values.constructor.name !== 'Array') {
      values = [values];
    }
    var count = 0;
    values.forEach(val => {
      if (eval(`"${val}" ${compareVal}`)) {
        count++;
      }
    });
    return count;
  }

  // 最小值
  funcMin(...values) {
    values = [].concat(...values); // flat
    values = removeNullParam(values);
    if (values.length === 0) { return null; }
    // 类型检查
    // checkValueType('number', 'MIN', 0, values);
    var min = values[0];
    for (var v of values) {
      if (min > v) {
        min = v;
      }
    }
    return min;
  }
  // 最大值
  funcMax(...values) {
    values = [].concat(...values); // flat
    values = removeNullParam(values);
    if (values.length === 0) { return null; }
    // 类型检查
    // checkValueType('number', 'MAX', 0, values);
    var max = values[0];
    for (var v of values) {
      if (max < v) {
        max = v;
      }
    }
    return max;
  }

  // 四舍五入
  funcRound(value, n) {
    checkParamCount('ROUND', 2, arguments);
    checkValueType('number', 'ROUND', 0, value, n);
    return Number(tofixed(Number(value),n));
  }

  // 向上取，保留n位小数
  funcRoundUp(value, n) {
    checkParamCount('ROUNDUP', 2, arguments);
    checkValueType('number', 'ROUNDUP', 0, value, n);
    var pow = Math.pow(10, n > value.decimalCnt() ? value.decimalCnt() : n);
    return Math.ceil(value * pow) / pow;
  }

  // 向下取，保留n位小数
  funcRoundDown(value, n) {
    checkParamCount('ROUNDDOWN', 2, arguments);
    checkValueType('number', 'ROUNDDOWN', 0, value, n);
    var pow = Math.pow(10, n > value.decimalCnt() ? value.decimalCnt() : n);
    return Math.floor(value.mul(pow)).div(pow);
  }

  // 取整数
  funcInt(value) {
    checkValueType('number', 'INT', 0, value);
    checkParamCount('INT', 1, arguments);
    return Math.floor(value);
  }
  // 取余（mod操作）
  funcMod(value, divisor) {
    checkValueType('number', 'MOD', 0, value, divisor);
    checkParamCount('MOD', 2, arguments);
    return Number(value) % Number(divisor);
  }
  // 连乘（product是office里面的称呼）
  funcProduct(...values) {
    values = [].concat(...values); // flat
    values = replaceNullParam(values);
    // 类型检查
    checkValueType('number', 'PRODUCT', 0, values);
    var result = 1;
    for (var v of values) {
      //   result *= Number.parseFloat(v);
      result = result.mul(Number.parseFloat(v));
    }
    return result;
  }
  // 对应相乘后相加
  funcSumProduct(value1, value2) {
    var result = 0;
    if (isNullOrUndefined(value1) || isNullOrUndefined(value2)) {
      return result;
    }
    if (value1.constructor.name !== 'Array') {
      value1 = [value1];
    }
    if (value2.constructor.name !== 'Array') {
      value2 = [value2];
    }
    // 获取两个数组的最小长度
    var length = value1.length < value2.length ? value1.length : value2.length;
    for (let index = 0; index < length; index++) {
      //   result += value1[index] * value2[index];
      result = result.add(Number.parseFloat(value1[index]).mul(Number.parseFloat(value2[index])));
    }
    return result;
  }
  // 对number求power次方
  funcPower(number, power) {
    checkValueType('number', 'POWER', 0, number, power);
    return Math.pow(number, power);
  }
  // 拿到年份
  funcYear(value) {
    checkParamCount('YEAR', 1, arguments);
    var t = moment(value, ['YYYY-MM-DD', 'YYYY-MM-DD HH:mm', 'YYYY-MM-DD HH:mm:ss']);
    return t.year();
  }
  // 拿到月份：注意，用moment拿月份时，是从0~11来计算的
  funcMonth(value) {
    checkParamCount('MONTH', 1, arguments);
    var t = moment(value, ['YYYY-MM-DD', 'YYYY-MM-DD HH:mm', 'YYYY-MM-DD HH:mm:ss']);
    return t.month() + 1;
  }
  // 拿到月份中的day
  funcDay(value) {
    checkParamCount('DAY', 1, arguments);
    var t = moment(value, ['YYYY-MM-DD', 'YYYY-MM-DD HH:mm', 'YYYY-MM-DD HH:mm:ss']);
    return t.date();
  }
  // 时
  funcHour(value) {
    checkParamCount('HOUR', 1, arguments);
    var t = moment(value, ['YYYY-MM-DD', 'YYYY-MM-DD HH:mm', 'YYYY-MM-DD HH:mm:ss']);
    return t.hour();
  }
  // 分
  funcMinite(value) {
    checkParamCount('MINITE', 1, arguments);
    var t = moment(value, ['YYYY-MM-DD', 'YYYY-MM-DD HH:mm', 'YYYY-MM-DD HH:mm:ss']);
    return t.minute();
  }
  // 秒
  funcSecond(value) {
    checkParamCount('SECOND', 1, arguments);
    var t = moment(value, ['YYYY-MM-DD', 'YYYY-MM-DD HH:mm', 'YYYY-MM-DD HH:mm:ss']);
    return t.second();
  }
  // 组装出日期，时分秒如果没有传递，替换成00
  funcDate(Y, M, D, H, m, s) {
    var t = moment();
    t.year(Number.parseInt(Y));
    t.month(Number.parseInt(M) - 1);
    t.date(Number.parseInt(D));
    t.hour(Number.parseInt(H ? H : 0));
    t.minute(Number.parseInt(m ? m : 0));
    t.second(Number.parseInt(s ? s : 0));
    return t.format('YYYY-MM-DD HH:mm:ss');
  }
  // 当前的日期
  funcCurDate() {
    var t = moment();
    return t.format('YYYY-MM-DD');
  }
  // 当前时间
  funcNow() {
    return moment().format('YYYY-MM-DD HH:mm:ss');
  }
  // 生成随机码
  funcRDID() {
    var d = new Date().getTime();
    if (typeof performance !== 'undefined' && typeof performance.now === 'function') {
      d += performance.now(); //use high-precision timer if available
    }
    return 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function (c) {
      var r = (d + Math.random() * 16) % 16 | 0;
      d = Math.floor(d / 16);
      return (c === 'x' ? r : (r & 0x3 | 0x8)).toString(16);
    });
  }

  // 把字符串中英文字符转换成大写
  funcUpper(value) {
    checkParamCount('UPPER', 1, arguments);
    return value.toString().toUpperCase();
  }

  // 把字符串中英文字符转换成小写
  funcLower(value) {
    checkParamCount('LOWER', 1, arguments);
    return value.toString().toLowerCase();
  }

  // 与操作
  funcAnd(...values) {
    values = [].concat(...values); // flat
    // 如果没有值的话，返回0
    if (values.length === 0) {
      return true;
    }
    // 类型检查
    checkValueType('boolean', 'AND', 0, values);
    return values.filter(val => val === true).length === values.length;
  }

  // 或操作
  funcOr(...values) {
    values = [].concat(...values); // flat
    // 如果没有值的话，返回0
    if (values.length === 0) {
      return true;
    }
    // 类型检查
    checkValueType('boolean', 'OR', 0, values);
    return !!values.find(val => val === true);
  }
  // 非操作
  funcNot(value) {
    // 类型检查
    checkValueType('boolean', 'NOT', 0, value);
    checkParamCount('NOT', 1, arguments);
    return !value;
  }
  // 异或操作
  funcXor(value1, value2) {
    // 类型检查
    checkValueType('boolean', 'XOR', 0, value1, value2);
    checkParamCount('XOR', 2, arguments);
    return value1 ^ value2 ? true : false;
  }

  // 检查传入的值是否为空
  funcIsEmpty(value) {
    if (isNullOrUndefined(value) || value === "") {
      return true; 
    } else if (value.constructor.name === 'Array') {
      if (value === [] || isNullOrUndefined(value[0]) || value[0].length === 0) {
        return true;
      }
    }
    return false;
  }

  // 查找字符串,在targetText中查找searchText所在位置，返回出现位置索引，没找到返回0,beginPos为开始查找的位置
  funcSearch(searchText, targetText, beginPos) {
    if (!beginPos) {
      beginPos = 1;
    }
    checkValueType('number', 'SEARCH', 2, beginPos);
    return String(targetText).indexOf(String(searchText), beginPos - 1) + 1;
  }

  // 获取字符串的长度
  funcLen(value) {
    if (isNullOrUndefined(value)) {
      return 0;
    } else if (value.constructor.name === 'Array') {
      return value.length;
    } else {
      return String(value).length;
    }
  }

  // 计算end和start之间相差的天数 传入参数为数组时，返回结果也会是数组
  funcDays(end, start) {
    checkParamCount('DAYS', 2, arguments);
    const dateMinus = (endDate, startDate) => {
      // 如果没有值的话，返回0
      if (isNullOrUndefined(endDate) || isNullOrUndefined(startDate)) {
        return 0;
      } else {
        return moment.duration(moment(endDate).unix() - moment(startDate).unix()).asDays() * 1000;
      }
    }

    if (Array.isArray(end) && Array.isArray(start)) {
      if (end.length === 0 && start.length === 0) {
        return 0;
      }
      const res = [];
      const arrLength = Math.max(end.length, start.length)
      for (let i = 0; i < arrLength; i++) {
        res.push(dateMinus(end[i], start[i]));
      }
      return res;
    } else {
      return dateMinus(end, start);
    }
  }

  // 计算出date增加或减少days天的日期（days可以为正数/负数）
  funcDateDelta(date, days) {
    checkValueType('number', 'DATEDELTA', 1, days);
    checkParamCount('DATEDELTA', 2, arguments);
    return moment(date).add(days, 'days').format("YYYY-MM-DD HH:mm:ss");
  }

  // 获取用户名，根据工作区备注>昵称>邮箱的优先级返回用户的用户名
  funcGetUserName(params) {
    if (isNullOrUndefined(params)) {
      return 0;
    }
    if (params['alias']) { return params['alias']; }
    if (params['nickName']) { return params['nickName']; }
    if (params['email']) { return params['email']; }
    return "";
  }

  // 返回用户的邮箱
  funcGetUserEmail(params) {
    if (isNullOrUndefined(params)) {
      return 0;
    }
    return params['email'];
  }

  // 获取ja学工号
  funJAaId(params) {
    if (isNullOrUndefined(params)) {
      return 0;
    }
    return params['jaInfo']['sid'] ? params['jaInfo']['sid'] : "";
  }

  // 获取ja姓名
  funJaName(params) {
    if (isNullOrUndefined(params)) {
      return 0;
    }
    if (params['nickName']) { return params['nickName']; }
    if (params['alias']) { return params['alias']; }
    if (params['email']) { return params['email']; }
    return "";
  }

  // 获取ja用户类型
  funJaType(params) {
    if (isNullOrUndefined(params)) {
      return 0;
    }
    let userType = params['jaInfo']['userType'];
    switch (userType) {
      case "student":
        return "学生";
      case "schoolFellow":
        return "校友";
      case "faculty":
        return "教职工";
      case "postphd":
        return "博士后";
      case "team":
        return "集体账号";
      default:
        return "其他";
    }
  }

  // 获取ja学院ID
  funJaDeptId(params) {
    if (isNullOrUndefined(params)) {
      return 0;
    }
    return params['jaInfo']['organizeId'];
  }

  // 获取ja学院名称
  funJaDeptName(params) {
    if (isNullOrUndefined(params)) {
      return '';
    }
    return params['jaInfo']['organize'];
  }

  // 获取表单打开次数
  funcRecno(params) {
    if (isNullOrUndefined(params)) {
      return 0;
    }
    return params['openCount'];
  }

  // 正则校验
  funcRegTest(text, reg, decorator) {
    var textList = [].concat(text);
    var result = true;
    textList.forEach(textTmp => {
      result = new RegExp(reg, decorator).test(textTmp);
    })
    return result;
  }

  // 求取Ln函数值
  funcLn(value) {
    checkValueType('number', 'LN', 0, value);
    checkParamCount('LN', 1, arguments);
    if (isNullOrUndefined(value)) {
      return null;
    }
    return Math.log(value);
  }

  // IF方法的升级版本
  funcIfs() {
    var i = 0;
    while (!isNullOrUndefined(arguments[i])) {
      if (arguments[i]) {
        return arguments[i + 1];
      }
      i += 2;
    }
    return null;
  }

  // base64编码
  funcBase64(value) {
    checkValueType('string', 'BASE64', 0, value);
    checkParamCount('BASE64', 1, arguments);
    if (isNullOrUndefined(value)) {
      return null;
    }
    return btoa(value);
  }

  // md5 加密
  funcMd5(val, encryption) {
    if (arguments.length > 2 || arguments.length < 1) {  // 只能传1个或2个参数
      checkParamCount('MD5', 2, arguments);
    }
    const v = Array.isArray(val) ? val[0] : val;
    if (isNullOrUndefined(v) || v.toString().length === 0) {
      return null;
    }
    const res = md5(v.toString()).toLowerCase();
    return encryption === 16 ? res.substr(8, 16) : res;
  }

  /**
   * TRIPLEDES
   * @param {*} value 加密内容
   * @param {*} inputKey 密钥：密钥长度必须是8的倍数，且大于等于24
   * @param {*} inputIv 向量：长度必须是8位，ECB模式下忽略偏移向量
   * @param {*} inputMode 模式
   * @param {*} inputPad 填充
   * @returns 
   */
  funTRIPLEDES(value, inputKey, inputIv, inputMode, inputPad){
    [value, inputKey, inputIv, inputMode, inputPad].forEach((e, i) => {
      if (e) {
        checkValueType('string', 'TRIPLEDES', i, e);
      }
    })
    const v= Array.isArray(value) ? value[0] : value;
    if(isNullOrUndefined(v) || v.toString().length ===0 ){
      return null;
    }
    if(inputKey && inputKey.length % 8 !== 0 && inputKey.length < 24){
      return 'Invalid Value';
    } 
    if(inputIv && inputIv.length !== 8){ 
      return 'Invalid Value';
    }
    const padObj= {"NOPADDING": CryptoJS.pad.NoPadding, "Iso10126": CryptoJS.pad.Iso10126};
    const pad = inputPad ? padObj[inputPad] : CryptoJS.pad.NoPadding;
    if(!pad){
      return 'Invalid Value';
    }
    const modeObj= {"ECB": CryptoJS.mode.ECB, "CBC": CryptoJS.mode.CBC, "CTR": CryptoJS.mode.CTR, "OFB": CryptoJS.mode.OFB, "CFB": CryptoJS.mode.CFB};
    const mode = inputMode ? modeObj[inputMode] : CryptoJS.mode.ECB;
    if(!mode || mode === CryptoJS.mode.CTR && pad !== CryptoJS.pad.NoPadding){
      return 'Invalid Value';
    }
    if (inputIv && ivs) {
      a.iv = ivs
    }
    const ivs = CryptoJS.enc.Utf8.parse(inputIv);
    const pads = {
      iv: ivs,
      mode: mode, 
      padding: pad
     }; 
    const key = CryptoJS.enc.Utf8.parse(inputKey);
    const encrypt = CryptoJS.TripleDES.encrypt(v, key, pads);
    return encrypt.toString();
  }


  // 求平方根
  funcSqrt(value) {
    checkValueType('number', 'SQRT', 0, value);
    checkParamCount('SQRT', 1, arguments);
    if (isNullOrUndefined(value)) {
      return null;
    }
    return Math.sqrt(value);
  }

  // TEXTUSER（成员字段,"name"/"email"/"phone"）
  // 获取所选成员在通讯录中的名称、邮箱或手机号。如：
  funcTextUser(detailInfo, values, mode) {
    checkValueType('string', 'TEXTUSER', 0, [mode]);
    if (isNullOrUndefined(values) || values.length === 0) {
      return "";
    }
    if (isNullOrUndefined(detailInfo) || isNullOrUndefined(detailInfo.memberInfos)) {
      return "";
    }
    switch (mode) {
      case 'email': {
        if (typeof values === 'string') {
          return values;
        }
        return values.join(',');
      }
      case 'phone':
      case 'name': {
        if (typeof values === 'string') {
          return detailInfo.memberInfos.has(values) ? detailInfo.memberInfos.get(values)[mode] : null;
        }
        return values
          .map(email => detailInfo.memberInfos.has(email) ? detailInfo.memberInfos.get(email)[mode] : null)
          .filter(v => !isNullOrUndefined(v))
          .join(',');
      }
      default: {
        return "";
      }
    }
  }

  // TEXTDEPT(部门字段,"name"/"id")
  // 获取所选部门的名称或ID。如：
  funcTextDept(detailInfo, values, mode) {
    checkValueType('string', 'TEXTDEPT', 0, [mode]);
    if (isNullOrUndefined(values) || values.length === 0) {
      return "";
    }
    if (isNullOrUndefined(detailInfo) || isNullOrUndefined(detailInfo.deptInfos)) {
      return "";
    }
    switch (mode) {
      case 'id': {
        if (typeof values === 'string' || typeof values === 'number') {
          return values;
        }
        return values.join(',');
      }
      case 'name': {
        if (typeof values === 'string' || typeof values === 'number') {
          return detailInfo.deptInfos.has(values) ? detailInfo.deptInfos.get(values)[mode] : null;
        }
        return values
          .map(deptId => detailInfo.deptInfos.has(deptId) ? detailInfo.deptInfos.get(deptId)[mode] : null)
          .filter(v => !isNullOrUndefined(v))
          .join(',');
      }
      default: {
        return "";
      }
    }
  }

  // 日期转时间戳
  funcDateToTimestamp(value) {
    checkValueType('string', 'DATETOTIMESTAMP', 0, value);
    checkParamCount('DATETOTIMESTAMP', 1, arguments);
    let fmt = SimpleDateFormat();
    let timestamp = fmt.parse(value).valueOf().toString();
    while (timestamp.length < 13) {
      timestamp = timestamp.concat('0')
    }
    return timestamp;
  }

  // 时间戳转日期
  funcTimestampToDate(value) {
    checkValueType('string', 'TIMESTAMPTODATE', 0, value);
    checkParamCount('TIMESTAMPTODATE', 1, arguments);
    if (value.length !== 10 && value.length !== 13) {
      return 'Invalid Value';
    } else if (value.length === 10) {
      value = value.concat('000');
    }
    var fmt = SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
    var date = fmt.format(value);
    return date;
  }

  /**
   * 计算工作日的天数
   * @param {日期字段1,日期字段2, weekend（周日到周六，1表示休息日）, 指定假期1, 指定假期2}
   */
  funcNetWorkdays(value) {
    if (isNullOrUndefined(value)) {
      value = '';
    }
    let [startDate, endDate, weekend, ...holidayArr] = arguments;
    const checkNumValue = removeNullParam([startDate, endDate]);
    checkParamCount('NETWORKDAYS', 2, checkNumValue);
    checkValueType('string', 'NETWORKDAYS', 0, ...arguments);
    if (startDate === '' && endDate === '') {
      return 0;
    }
    startDate = moment(startDate, 'YYYY-MM-DD').format('YYYY-MM-DD');
    endDate = moment(endDate, 'YYYY-MM-DD').format('YYYY-MM-DD');
    holidayArr.map(e => moment(e, 'YYYY-MM-DD').format('YYYY-MM-DD'))
    // weekend只能是含有0和1的7位字符串
    if (!isNullOrUndefined(weekend)) {
      checkReg(weekend, '^[0-1]{7}$', 'NETWORKDAYS', 3);
    } else {
      weekend = '1000001';
    }
    // 判断所有日期是否有效
    try {
      checkDateValid(startDate, 'NETWORKDAYS', 1);
      checkDateValid(endDate, 'NETWORKDAYS', 2);
      checkDateValid(holidayArr, 'NETWORKDAYS', 4);
    } catch (e) {
      return null;
    }
    // 判断startDate和endDate的大小并记录
    let prefix = 1;
    if (moment(startDate).isAfter(endDate, 'day')) {
      [startDate, endDate] = [endDate, startDate];
      prefix = -1;
    }
    // 日期差额
    const allDay = moment(endDate).diff(moment(startDate), 'days') + 1;
    // 获取休息日下标（weekend为1）
    let weekendIndex = [];
    weekend.split('').forEach((e, i) => {
      if (e === '1') {
        weekendIndex.push(i);
      }
    })
    // 分为三部分计算休息时间的天数，开始、中间完整周、结尾
    const start =  moment(startDate, "YYYY-MM-DD", true);
    const end =  moment(endDate, "YYYY-MM-DD", true);
    const first =  moment(startDate).clone().endOf('week'); //第一周的结束
    const last =  moment(endDate).clone().startOf('week'); // 最后一周的开始
    let midWeekendNum = 0;
    let seWeekendNum = 0;
    if (last.isAfter(first, 'day')) {
      midWeekendNum = parseInt(last.diff(first,'days') * weekendIndex.length / 7, 10); // 中间完整周的休息天数
      for (let i = start.day(); i <= end.day() + 7; i++) { // 首尾周的休息时间
        if (weekendIndex.includes(i % 7)) {
          seWeekendNum ++;
        }
      }
    } else {
      for (let i = start.day(); i <= end.day(); i++) { // 首尾周的休息时间
        if (weekendIndex.includes(i % 7)) {
          seWeekendNum ++;
        }
      }
    }
    // 区间内指定假期
    let betweenHolidayNum = 0;
    holidayArr.forEach((holiday, i) => {
      const holidate = moment(holiday, "YYYY-MM-DD", true);
      const isBetweenOrSame = holidate.isBetween(startDate, endDate, 'day') || holidate.isSame(startDate, 'day') || holidate.isSame(endDate, 'day');
      const repeatIndex = holidayArr.indexOf(holiday, i + 1);
      if (holidate.isValid() && isBetweenOrSame  && !weekendIndex.includes(holidate.day()) && repeatIndex === -1) {
        betweenHolidayNum ++
      }
    });
    const weekDay = allDay - midWeekendNum - seWeekendNum - betweenHolidayNum;
    return prefix * weekDay;
  }

  funcUnicodeEncode(value) {
    checkParamCount('UNICODEENCODE', 1, arguments);
    var encodeStr = value;
    if (Array.isArray(value)) {
      encodeStr = value.join('');
    }
    if (!encodeStr && encodeStr !== 0) {
      return null;
    }
    encodeStr = encodeStr.toString();
    return encodeStr.split('').map(function (str, index, array) {
      var temp = encodeStr.charCodeAt(index).toString(16).padStart(4, '0');
      if (temp.length > 2) {
        return '\\u' + temp;
      }
      return str;
    }).join('');
  }

  funcUnicodeDecode(value) {
    checkParamCount('UNICODEDECODE', 1, arguments);
    var temp = value;
    if (Array.isArray(value)) {
      temp = value.join('');
    }
    if (!temp && temp !== 0) {
      return null;
    }
    temp = temp.toString();
    return temp.replace(/\\u([0-9a-fA-F]{4})/g, (content, $1) => {
      return String.fromCharCode(parseInt($1,16));
    })
  }
}
